#!/usr/bin/env python3
"""
Spacing - Python Blank Line Formatter
Can be run directly as src/spacing.py or installed as `spacing` command.

Copyright (c) 2025-2026 Greg Smethells. All rights reserved.
See the accompanying AUTHORS file for a complete list of authors.
This file is subject to the terms and conditions defined in LICENSE.
"""

from spacing.cli import main

if __name__ == '__main__':
  main()
